package com.qs.mp.web.controller.api.admin;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qs.mp.admin.domain.Coupon;
import com.qs.mp.admin.domain.Goods;
import com.qs.mp.admin.domain.GoodsCategory;
import com.qs.mp.admin.domain.param.CouponParam;
import com.qs.mp.admin.domain.param.GoodsParam;
import com.qs.mp.admin.domain.vo.GoodsListVO;
import com.qs.mp.admin.service.IGoodsCategoryService;
import com.qs.mp.admin.service.IGoodsService;
import com.qs.mp.admin.service.IGoodsSkuService;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.domain.param.ChannelParam;
import com.qs.mp.channel.domain.vo.ChannelVO;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.channel.service.IChannelUserRelService;
import com.qs.mp.common.constant.UserConstants;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.ChannelRoleEnum;
import com.qs.mp.common.enums.CouponStatusEnum;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.system.domain.SysUser;
import com.qs.mp.system.service.ISysUserService;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import ma.glasnost.orika.MapperFacade;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @auther liugl
 * @create 2022-03-09 23:45:48
 * @describe 商品管理前端控制器
 */
@Api("渠道管理API")
@RestController
@RequestMapping("/api/v1/mp/admin/goods/*")
@Component
public class GoodsMgrController extends BaseApiController {

	@Autowired
	private IGoodsService goodsService;
	
	@Autowired
	private IGoodsSkuService goodsSkuService;
	
	@Autowired
	private IGoodsCategoryService goodsCategoryService;
	
	@Autowired
	private ISysUserService sysUserService;

	@Autowired
	private MapperFacade mapperFacade;


	/**
	 * 查询商品列表, 支持翻页
	 *
	 * @return
	 */
	@PostMapping("/list")
	public TableDataInfo list(@RequestBody Goods goods) {
		List<GoodsListVO> list = new ArrayList<GoodsListVO>();
		startPage();
		List<Goods> goodsList = goodsService.list(new LambdaQueryWrapper<Goods>());
		List<GoodsListVO> goodsListVOList = mapperFacade.mapAsList(goodsList, GoodsListVO.class);
		TableDataInfo res = getDataTable(goodsList);
		res.setRows(goodsListVOList);
		return res;
	}


	/**
	 * 新增商品信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "新增商品信息", notes = "后台商品管理新增商品")
	@PostMapping("/create")
	public AjaxResult channelCreate(@Validated @RequestBody GoodsParam goodsParam) {
		if (goodsParam.getGoodsId() != null && goodsParam.getGoodsId() != 0) {
			return AjaxResult.error("该商品已存在");
		}
		Goods goods = mapperFacade.map(goodsParam, Goods.class);
		// 1、校验名称是否重复（商品表）
		LambdaQueryWrapper<Goods> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.eq(Goods::getTitle, goods.getTitle());
		int titleCount = goodsService.count(queryWrapper);
		if(titleCount > 0) {
			return AjaxResult.error("商品名称" + goods.getTitle() + "已存在！");
		}
		
		// 多SKU
		if(goods.getMultiSku() == 1 
				&&  (null == goodsParam.getSkuList() || goodsParam.getSkuList().size() == 0)) {
			return AjaxResult.error("商品" + goods.getTitle() + "不满足多SKU条件！");
		}
		// 3.插入数据
		try {
			goodsService.saveGoods(goods, goodsParam.getSkuList());
		} catch (Exception e) {
			return AjaxResult.error("商品'" + goods.getTitle() + "'新增失败" + e.getMessage());
		}
		return AjaxResult.success("商品'" + goods.getTitle() + "'新增成功");
	}

	/**
	 * 编辑子渠道信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "编辑商品信息", notes = "后台商品管理修改商品信息")
	@PostMapping("/update")
	public AjaxResult channelUpdate(@Validated @RequestBody GoodsParam goodsParam) {
		
		if (goodsParam.getGoodsId() != null && goodsParam.getGoodsId() != 0) {
			return AjaxResult.error("该商品已存在");
		}
		Goods oldGoods = goodsService.getById(goodsParam.getGoodsId());
		if(null == oldGoods || null == oldGoods.getGoodsId()) {
			return AjaxResult.error("商品'" + oldGoods.getTitle() + "'编辑失败，商品ID异常");
		}
		Goods goods = mapperFacade.map(goodsParam, Goods.class);
		// 1、校验名称是否重复（商品表）
		if(!goods.getTitle().equals(oldGoods.getTitle())) {
			LambdaQueryWrapper<Goods> queryWrapper = new LambdaQueryWrapper<>();
			queryWrapper.eq(Goods::getTitle, goods.getTitle());
			int titleCount = goodsService.count(queryWrapper);
			if(titleCount > 0) {
				return AjaxResult.error("商品名称" + goods.getTitle() + "已存在！");
			}
		}
		
		// 多SKU
		if(goods.getMultiSku() == 1 
				&&  (null == goodsParam.getSkuList() || goodsParam.getSkuList().size() == 0)) {
			return AjaxResult.error("商品" + goods.getTitle() + "不满足多SKU条件！");
		}
		// 3.插入数据
		try {
			goodsService.updateGoods(goods, goodsParam.getSkuList());
		} catch (Exception e) {
			return AjaxResult.error("商品'" + goods.getTitle() + "'新增失败" + e.getMessage());
		}
		return AjaxResult.success("商品'" + goods.getTitle() + "'新增成功");
	}

	
	
	/**
	 * 停用、启用商品
	 * @param
	 * @return
	 */
	@ApiOperation(value = "停用、启用代金券信息", notes = "代金券管理编辑代金券")
	@PostMapping("/status")
	public AjaxResult couponStatus(@RequestBody Goods goods) {
		String goodsId = (null != goods && StringUtils.isNotBlank(goods.getGoodsId()))?goods.getGoodsId():"";
		String status = (null != goods && StringUtils.isNotBlank(goods.getStatus()))?goods.getStatus():"";
		if (StringUtils.isBlank(goodsId)
				|| StringUtils.isBlank(status)) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		try {
			goodsService.lambdaUpdate().set(Goods::getStatus, goods.getStatus()).eq(Goods::getGoodsId, goodsId).update();
			// 查询代金券信息
		} catch (Exception e) {
			return AjaxResult.error("操作失败");
		}
		return AjaxResult.success("操作成功");
	}

}
