package com.qs.mp.admin.service.impl;

import com.qs.mp.admin.domain.Goods;
import com.qs.mp.admin.domain.GoodsSku;
import com.qs.mp.admin.mapper.GoodsMapper;
import com.qs.mp.admin.service.IGoodsService;
import com.qs.mp.admin.service.IGoodsSkuService;
import com.qs.mp.common.exception.ServiceException;
import com.qs.mp.system.service.id.BizIdGenerator;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 商品库 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-07
 */
@Service
public class GoodsServiceImpl extends ServiceImpl<GoodsMapper, Goods> implements IGoodsService {

	@Autowired
	private IGoodsService goodsService;
	
	@Autowired
	private IGoodsSkuService goodsSkuService;
	
    @Autowired
    private BizIdGenerator bizIdGenerator;
	
	@Override
	@Transactional
	public void saveGoods(Goods goods, List<GoodsSku> skuList) {
		goods.setGoodsId(bizIdGenerator.newId());
		boolean res = goodsService.save(goods);
		if(!res) {
			throw new ServiceException("请联系管理员");
		}
		if(res && null != skuList
				&& skuList.size() > 0) {
			for(GoodsSku goodsSku:skuList) {
				if(null != goodsSku) {
					goodsSku.setGoodsId(goods.getGoodsId());
				}
			}
			boolean skuRes = goodsSkuService.saveBatch(skuList);
			if(!skuRes) {
				throw new ServiceException("请联系管理员");
			}
		}
	}

	@Override
	@Transactional
	public void updateGoods(Goods goods, List<GoodsSku> skuList) {
		boolean res = goodsService.updateById(goods);
		if(!res) {
			throw new ServiceException("请联系管理员");
		}
		// 多SKU
		if(res && null != skuList
				&& skuList.size() > 0) {
			for(GoodsSku goodsSku:skuList) {
				if(null != goodsSku) {
					goodsSku.setGoodsId(goods.getGoodsId());
				}
			}
			// TODO 未完待续
			boolean skuRes = goodsSkuService.saveBatch(skuList);
			if(!skuRes) {
				throw new ServiceException("请联系管理员");
			}
		}
	}
}
